MODULE calculation
	implicit none
	PRIVATE
	PUBLIC:: area, triangularNumb, triNumb_rec, fac_rec, cuboid, operator(.EXPO.) &
	& operator (.EXPOrec.), operator(.BINrec.), operator(.BIN.)

	INTERFACE OPERATOR (.BINrec.)
		MODULE PROCEDURE binCo_rec
	END INTERFACE
	
	INTERFACE OPERATOR (.BIN.)
		MODULE PROCEDURE binCo
	END INTERFACE
	
	 INTERFACE OPERATOR (.EXPO.)
		MODULE PROCEDURE expon_it
	END INTERFACE
	
	 INTERFACE OPERATOR (.EXPOrec.)
		MODULE PROCEDURE expon_rec
	END INTERFACE

	CONTAINS

		FUNCTION area(d)
			REAL, INTENT(IN):: d
			REAL:: area, pi = 3.14159265359
			area = pi*d**2/4
		END FUNCTION 

		FUNCTION binCo(n, m)
			INTEGER, INTENT(IN):: n, m
			INTEGER:: binCo, i
		 	binCo = 1
			DO i = 1, min(m,n-m)
				binCo = binCo*(n-i+1)/i
			END DO
		END FUNCTION

		FUNCTION expon_it(x, y)
			REAL, INTENT(IN):: x
			REAL:: expon_it
			INTEGER, INTENT(IN):: y
			INTEGER:: i
			expon_it = 1
			DO i = 1, y
				expon_it = expon_it * x
			END DO
		END FUNCTION expon_it

		RECURSIVE FUNCTION expon_rec(a,b) RESULT (res)
			INTEGER, INTENT(IN):: b
			REAL, INTENT(IN):: a
			REAL:: x, res
			IF (b == 1) THEN
				res = a
			ELSE IF (MOD(b,2)== 0) THEN
				x = expon_rec(a, b/2)
				res = x*x
			ELSE 
				x = expon_rec(a, b/2)
				res = x*x * a
			END IF
		END FUNCTION expon_rec
		
		RECURSIVE FUNCTION binCo_rec(x, y) RESULT (res)
			INTEGER, INTENT(IN):: x, y 
			INTEGER:: res
			IF (y > x) THEN
				res = 0
			ELSE IF (x == y.OR. y == 0) THEN
				res = 1
			ELSE IF (y == 1.OR. y == x-1) THEN
				res = x
			ELSE 
				res = binCo_rec(x-1, y-1) + binCo_rec(x-1, y)
			END IF
		END FUNCTION

		FUNCTION triangularNumb(p)
			INTEGER, INTENT(IN):: p
			INTEGER:: triangularNumb 
			triangularNumb = (p*(p+1))/2
		END FUNCTION 

		RECURSIVE FUNCTION triNumb_rec(x) RESULT(res)
			INTEGER, INTENT(IN):: x
			INTEGER:: res
			IF (x==0) THEN
				res = 0
			ELSE 
				res = res + fac_rec(x-1)
			END IF
		END FUNCTION

		RECURSIVE FUNCTION fac_rec(x) RESULT (res)
			INTEGER, INTENT(IN):: x
			INTEGER:: res
			IF (x==0.OR. x==1) THEN
				res = 1
			ELSE 
				res = res*fac_rec(x-1)
			END IF
		END FUNCTION

		SUBROUTINE cuboid(a, b, h, Vol, Ob)
				REAL, INTENT(IN):: a, b, h
				REAL, INTENT(OUT):: Vol, Ob
				Vol = a*b*h
				Ob = 2*(a*b + a*h + b*h)
		END SUBROUTINE

END MODULE calculation

PROGRAM calculator
	USE calculation
	IMPLICIT NONE

	REAL:: h, x, y, z, Vol, Ob
	!REAL, PARAMETER :: real_kind= SELECTED_REAL_KIND (P=18, R=4931)
	CHARACTER:: c
	INTEGER:: n, m
	
	DO
		WRITE(*,*)
		WRITE(*,*) 'Please enter calculation operation (+,-,*,:, "b" for binomial coefficient, "c" for cirlce area, "e" for exponent, ' 		
		WRITE(*,*) '"f" faculty,"q" for prism volume and surface area, or "t" for triangular number):'
		READ(*,*) c
		WRITE(*,*)

	IF(c == 'b') THEN
		WRITE(*,*) 'Please enter integer values for n and m:'
		READ(*,*) n, m
	ELSE IF(c == 'c') THEN
		WRITE(*,*) 'Please enter real values for diameter:'
		READ(*,*) x
	ELSE IF(c == 'e') THEN
		WRITE(*,*) 'Please enter real value for basis and integer value for exponent:'
		READ(*,*) x, m
	ELSE IF(c == 'f') THEN
		WRITE(*,*) 'Please enter integer value for faculty calculation:'
		READ(*,*) m
	ELSE IF(c == 'q') THEN
		WRITE(*,*) 'Please enter real values for a, b and height h:'
		READ(*,*) x, y, h
	ELSE IF(c == 't') THEN
		WRITE(*,*) 'Please enter integer value triangular number:'
		READ(*,*) n
	ELSE IF (c == '+'.OR.c=='-'.OR.c=='*'.OR.c==':') THEN
		WRITE(*,*) 'Please enter real values for x and y:'
		READ(*,*) x, y 
	ELSE
		WRITE(*,*) 'No permitted calculation operation entered.'
	END IF

		SELECT CASE(c)
			CASE('+') 
				z = x + y
				WRITE(*,*) x ,'+', y, '=', z
			CASE('-')
				z = x - y
				WRITE(*,*) x ,'-', y, '=', z
			CASE('*')
				z = x*y
				WRITE(*,*) x ,'*', y, '=', z
			CASE(':')
				IF (y/=0) THEN
					z = x/y
					WRITE(*,*) x ,':', y, '=', z
				ELSE
					WRITE(*,*) 'Dividing by zero is not permitted.'
				END IF
			CASE('b')
				WRITE(*,*) 'The binomial coeffient is', n.BIN.m
				WRITE(*,*) 'Recursively calculated:', n.BINrec.m
			CASE('c')
				WRITE(*,*) 'The area is', area(x)
			CASE('e')
				WRITE(*,*) 'The result is', x**m
				WRITE(*,*) 'The iterative calculated result is', x.EXPO.m
				WRITE(*,*) 'The recursively calculated result is', x.EXPOrec.m
			CASE('f')
				WRITE(*,*) 'The faculty of', m,'is', fac_rec(m)
			CASE('q')
				CALL cuboid(x, y, h, Vol, Ob)
				WRITE(*,*) 'Volumen of the cuboid:', Vol
				WRITE(*,*) 'Area of the cuboid:', Ob
			CASE('t')
				WRITE(*,*) 'The triangular number is', triangularNumb(n)
				WRITE(*,*) 'Recursively calculated:', triNumb_rec(n)
			CASE DEFAULT 
				EXIT
		END SELECT
	END DO

END PROGRAM calculator
